<?php

class ArrayList extends __prop_base_class implements IteratorAggregate , ArrayAccess , Serializable , Countable 
{
	// 定数 {
		// 継承したクラスでこの定数を再定義することで、型指定付きの配列を作ることが出来る。
		const VALUE_TYPE = null;
		
		// 継承したクラスでこの定数を再定義することで、arrayの自動変換を抑制できる。
		const ARRAY_CONVERT = true;
		
		const ERROR_TYPE_WARNING = 0;
		const ERROR_TYPE_ERROR = 1;
		const ERROR_TYPE_EXCEPTION = 2;
		static $ERROR_TYPE = self::ERROR_TYPE_EXCEPTION;
	// }
	
	// メンバ {
		protected $_storage = array();
		protected $_lastError;
		protected $_lastErrorMessage;
	// }
	
	// プロパティ {
		protected $_prop_lastError = self::PROPERTY_READONLY;
		protected $_prop_lastErrorMessage = self::PROPERTY_READONLY;
	// }
	
	// Iterator の実装 {
		/*
		protected $_currentPos = 0;
		public function current() {
			return $this->getAt($this->_currentPos);
		}
		public function key() {
			return $this->getKeyAt($this->_currentPos);
		}
		public function next() {
			$this->_currentPos++;
		}
		public function rewind() {
			$this->_currentPos = 0;
		}
		public function valid() {
			return $this->existsAt($this->_currentPos);
		}
		*/
	// }
	
	// IteratorAggregate の実装 {
		public function getIterator() {
			return new ArrayIterator($this->_storage);
		}
	// }
	
	// ArrayAccess の実装 {
		public function offsetExists($offset) {
			return $this->existsKey($offset);
		}
		public function offsetGet($offset) {
			return $this->get($offset);
		}
		public function offsetSet($offset, $value) {
			if(is_null($offset))
				$this->append($value);
			else
				$this->set($offset, $value);
		}
		public function offsetUnset($offset) {
			return $this->remove($offset);
		}
	// }
	
	// Serializable の実装 {
		public function serialize() {
			return serialize($this->_storage);
		}
		public function unserialize($data) {
			$this->exchangeArray(unserialize($data));
		}
	// }
	
	// Countable の実装 {
		public function count() {
			return count($this->_storage);
		}
	// }
	
	// 静的メソッド {
		static public function checkVariableType(&$v) {
			if(! is_null(static::VALUE_TYPE)) {
				switch(static::VALUE_TYPE) {
				case 'int':
				case 'integer':
				case 'float':
				case 'double':
					if(! is_numeric($v)) return false;
					settype($v, static::VALUE_TYPE);
					break;
				case 'string':
				case 'bool':
				case 'boolean':
					settype($v, static::VALUE_TYPE);
					break;
				case 'array':
					if(static::ARRAY_CONVERT && gettype($v) == 'object' && ($v instanceof ArrayList)) break;
				case 'object':
					if(gettype($v) != static::VALUE) return false;
					break;
				default:
					$class = static::VALUE_TYPE;
					if(! ($v instanceof $class)) return false;
					break;
				}
			}
			
			if(static::ARRAY_CONVERT && is_array($v)) {
				$v = new ArrayList($v);
			}
			
			return true;
		}
	// }
	
	// コンストラクタ {
		public function __construct(array $arr = array()) {
			$this->exchangeArray($arr);
		}
	// }
	
	// パブリックメソッド {
		public function getArrayCopy() {
			return $this->getData(true);
		}
		public function getData($copy = false) {
			if($copy) {
				$ret = array();
				foreach($this->_storage as $k=>$v) {
					$ret[$k] = $this->get($k);
				}
				return $ret;
			}
			
			return $this->_storage;
		}
		
		public function exchangeArray(array $arr) {
			foreach($arr as $k=>$v) {
				if(! static::checkVariableType($arr[$k]))
					return $this->_retError(false, __LINE__, '%s class must be contains %s. %s given.', get_class($this), static::VALUE_TYPE, gettype($v)!='object' ? gettype($v) : get_class($v));
			}
			$this->_storage = $arr;
			return true;
		}
		
		// 以下のメソッドは失敗したときにnullを返しますが、null値が配列に含まれている場合もあります。
		// 値としてnullを返したのか、失敗したのかは、以下のいずれかの方法で検証可能です。
		// ・失敗時にnullを返すメソッド
		//   current, key, get, getAt, getKeyAt
		// ・検証方法
		//   if($arrayList->issetAt($index)) { $ret = $arrayList->getAt($index); } else { /* 失敗時の処理 */ }
		//   if(($ret = $arrayList->getAt($index)) === null && $arrayList->lastError) { /* 失敗時の処理 */ }
		public function existsAt($index) {
			$this->_lastError = 0;
			
			$keys = array_keys($this->_storage);
			return isset($keys[$index]);
		}
		public function existsKey($key) {
			$this->_lastError = 0;
			
			return isset($this->_storage[$key]);
		}
		public function exists($val, $strict = false) {
			return $this->getKey($val, $strict) !== false;
		}
		
		public function getAt($index) {
			return $this->get($this->getKeyAt($index));
		}
		public function getKeys() {
			return array_keys($this->_storage);
		}
		public function getKey($val, $strict = false) {
			$this->_lastError = 0;
			
			return array_search($val, $this->_storage, $strict);
		}
		public function getKeyAt($index) {
			$this->_lastError = 0;
			
			$keys = $this->getKeys();
			if(! isset($keys[$index])) return $this->_retError(null, __LINE__, 'Index %d is undefined.', $index);
			
			return $keys[$index];
		}
		public function get($key) {
			$this->_lastError = 0;
			
			if(! isset($this->_storage[$key])) return $this->_retError(null, __LINE__, 'Key %s is undefined.', $key);
			
			return $this->_storage[$key];
		}
		public function getIndex($key) {
			$keys = $this->getKeys();
			$index = array_search($key, $keys);
			if($index === false) return $this->_retError(-1, __LINE__, 'Key %s is not found.', $key);
			
			return $index;
		}
		
		public function setAt($index, $value) {
			$this->_lastError = 0;
			
			if(! $this->existsAt($index)) {
				$this->append($value);
			} else {
				$key = $this->getKeyAt($index);
				$this->set($key, $value);
			}
			
			return true;
		}
		public function setKeyAt($index, $key) {
			$this->_lastError = 0;
			
			if(! $this->existsAt($index))
				return $this->_retError(false, __LINE__, 'Index %d is undefined.', $index);
			if($this->exists($key) && $this->getIndex($key) != $index)
				return $this->_retError(false, __LINE__, 'Key %s is duplicated.', $key);
			
			$keys = $this->getKeys();
			$new = array();
			foreach($keys as $i => $okey) {
				if($i == $index) $okey = $key;
				$new[$key] = $this->_storage[$okey];
			}
			
			$this->_storage = $new;
			return true;
		}
		public function set($key, $value) {
			$this->_lastError = 0;
			if(! static::checkVariableType($value))
				return $this->_retError(false, __LINE__, '%s class must be contains %s. %s given.', get_class($this), static::VALUE_TYPE, gettype($value)!='object' ? gettype($value) : get_class($value));
			
			if(is_null($key))
				$this->_storage[] = $value;
			else
				$this->_storage[$key] = $value;
			
			return true;
		}
		public function append($value) {
			return $this->set(null, $value);
		}
		public function add($value) {
			return $this->append($value);
		}
		
		public function remove($key) {
			unset($this->_storage[$key]);
		}
		public function removeAt($index) {
			return $this->remove($this->getKeyAt($index));
		}
		
		public function sort() {
			$ret = clone($this);
			sort($ret->_storage);
			return $ret;
		}
		public function rsort() {
			$ret = clone($this);
			rsort($ret->_storage);
			return $ret;
		}
		public function asort() {
			$ret = clone($this);
			asort($ret->_storage);
			return $ret;
		}
		public function arsort() {
			$ret = clone($this);
			arsort($ret->_storage);
			return $ret;
		}
		public function ksort() {
			$ret = clone($this);
			ksort($ret->_storage);
			return $ret;
		}
		public function krsort() {
			$ret = clone($this);
			krsort($ret->_storage);
			return $ret;
		}
		public function natsort() {
			$ret = clone($this);
			natsort($ret->_storage);
			return $ret;
		}
		public function natcasesort() {
			$ret = clone($this);
			natcasesort($ret->_storage);
			return $ret;
		}
		public function reverse() {
			$ret = clone($this);
			$ret->_storage = array_reverse($ret->_storage);
			return $ret;
		}
		public function shuffle() {
			$ret = clone($this);
			shuffle($ret->_storage);
			return $ret;
		}
	// }
	
	// 内部メソッド {
		protected function _retError($ret, $code, $message = '') {
			$this->_lastError = $code;
			$p = func_Get_args();
			array_shift($p);
			array_shift($p);
			$this->_lastErrorMessage = call_user_func_array('sprintf', $p);
			
			switch(static::$ERROR_TYPE) {
			case self::ERROR_TYPE_WARNING:
				trigger_error($this->_lastErrorMessage, E_USER_WARNING);
				break;
			case self::ERROR_TYPE_ERROR:
				trigger_error($this->_lastErrorMessage, E_USER_ERROR);
				break;
			case self::ERROR_TYPE_EXCEPTION:
				throw new Exception($this->_lastErrorMessage);
				break;
			}
			return $ret;
		}
		/**
		 * var_dumpやprint_rされる際に、列挙するプロパティの名前を配列で返す。
		 */
		protected function __var_dump($ret) {
			$ret = parent::__var_dump($ret);
			if(($i = array_search('ERROR_TYPE', $ret)) !== false) unset($ret[$i]);
			return $ret;
		}
	// }
}

class PermissionArrayList extends ArrayList
{
	// 定数 {
		const PERMISSION_ALL		= 0xFFFFFFFF;
		const PERMISSION_READ		= 0x00000001;
		const PERMISSION_WRITE		= 0x00000002;
		const PERMISSION_OVERWRITE	= 0x00000004;
		const PERMISSION_UNSET		= 0x00000008;
		const PERMISSION_ITERATION	= 0x00000010;
	// }
	// メンバ {
		protected $_permission = self::PERMISSION_ALL;
	// }
	
	// コンストラクタ {
		public function __construct(array $arr = array(), $permissions = self::PERMISSION_ALL) {
			parent::__construct($arr);
			$this->_permission = $permissions;
		}
	// }
	
	// IteratorAggregate の実装 {
		public function getIterator() {
			if(! $this->_checkPermission(__LINE__, self::PERMISSION_ITERATION))
				return new IteratorAggregate(array());
			
			$p = func_get_args(); return call_user_func_array('parent::'.__FUNCTION__, $p);
		}
	// }
	
	// Countable の実装 {
		public function count() {
			if(! $this->_checkPermission(__LINE__, self::PERMISSION_READ))
				return false;
			
			$p = func_get_args(); return call_user_func_array('parent::'.__FUNCTION__, $p);
		}
	// }
	
	// パブリックメソッド {
		public function getData($copy = false) {
			if(! $this->_checkPermission(__LINE__, self::PERMISSION_READ))
				return false;
			
			$p = func_get_args(); return call_user_func_array('parent::'.__FUNCTION__, $p);
		}
		
		public function exchangeArray(array $arr) {
			if(! $this->_checkPermission(__LINE__, self::PERMISSION_WRITE | self::PERMISSION_OVERWRITE))
				return false;
			
			$p = func_get_args(); return call_user_func_array('parent::'.__FUNCTION__, $p);
		}
		
		public function existsAt($index) {
			if(! $this->_checkPermission(__LINE__, self::PERMISSION_READ))
				return false;
			
			$p = func_get_args(); return call_user_func_array('parent::'.__FUNCTION__, $p);
		}
		public function exists($key) {
			if(! $this->_checkPermission(__LINE__, self::PERMISSION_READ))
				return false;
			
			$p = func_get_args(); return call_user_func_array('parent::'.__FUNCTION__, $p);
		}
		
		public function getAt($key) {
			if(! $this->_checkPermission(__LINE__, self::PERMISSION_READ | self::PERMISSION_ITERATION))
				return false;
			
			$p = func_get_args(); return call_user_func_array('parent::'.__FUNCTION__, $p);
		}
		public function getKeys() {
			if(! $this->_checkPermission(__LINE__, self::PERMISSION_READ))
				return false;
			
			$p = func_get_args(); return call_user_func_array('parent::'.__FUNCTION__, $p);
		}
		public function getKeyAt($index) {
			if(! $this->_checkPermission(__LINE__, self::PERMISSION_READ | self::PERMISSION_ITERATION))
				return false;
			
			$p = func_get_args(); return call_user_func_array('parent::'.__FUNCTION__, $p);
		}
		public function get($key) {
			if(! $this->_checkPermission(__LINE__, self::PERMISSION_READ))
				return false;
			
			$p = func_get_args(); return call_user_func_array('parent::'.__FUNCTION__, $p);
		}
		public function getIndex($key) {
			if(! $this->_checkPermission(__LINE__, self::PERMISSION_READ | self::PERMISSION_ITERATION))
				return false;
			
			$p = func_get_args(); return call_user_func_array('parent::'.__FUNCTION__, $p);
		}
		
		public function setAt($index, $value) {
			if(! $this->_checkPermission(__LINE__, self::PERMISSION_WRITE | self::PERMISSION_ITERATION))
				return false;
			
			$p = func_get_args(); return call_user_func_array('parent::'.__FUNCTION__, $p);
		}
		public function setKeyAt($index, $key) {
			if(! $this->_checkPermission(__LINE__, self::PERMISSION_READ | self::PERMISSION_ITERATION))
				return false;
			
			if(! $this->existsAt($index)) {
				if(! $this->_checkPermission(__LINE__, self::PERMISSION_WRITE))
					return false;
			} else {
				if(! $this->_checkPermission(__LINE__, self::PERMISSION_OVERWRITE))
					return false;
			}
			
			$p = func_get_args(); return call_user_func_array('parent::'.__FUNCTION__, $p);
		}
		public function set($key, $value) {
			if(! isset($this->_storage[$key])) {
				if(! $this->_checkPermission(__LINE__, self::PERMISSION_WRITE))
					return false;
			} else {
				if(! $this->_checkPermission(__LINE__, self::PERMISSION_OVERWRITE))
					return false;
			}
			
			$p = func_get_args(); return call_user_func_array('parent::'.__FUNCTION__, $p);
		}
		public function append($value) {
			if(! $this->_checkPermission(__LINE__, self::PERMISSION_WRITE))
				return false;
			
			$p = func_get_args(); return call_user_func_array('parent::'.__FUNCTION__, $p);
		}
		
		public function remove($key) {
			if(! $this->_checkPermission(__LINE__, self::PERMISSION_UNSET))
				return false;
			
			$p = func_get_args(); return call_user_func_array('parent::'.__FUNCTION__, $p);
		}
		
		public function shift($num = 1) {
			for($i = 0; $i < $num; $i++) {
				$ret = $this->getAt(0);
				$this->removeAt(0);
			}
			
			return $ret;
		}
		public function pop($num = 1) {
			for($i = 0; $i < $num; $i++) {
				$ret = $this->getAt($this->count() - 1);
				$this->removeAt($this->count() - 1);
			}
			
			return $ret;
		}
	// }
	
	// 内部メソッド {
		protected function _checkPermission($code, $permission) {
			if(($this->_permission & $permission) != $permission) {
				$perms = array();
				if(($permission & self::PERMISSION_ITERATION) != 0) $perms[] = 'ITERATION';
				if(($permission & self::PERMISSION_OVERWRITE) != 0) $perms[] = 'OVERWRITE';
				if(($permission & self::PERMISSION_READ) != 0) $perms[] = 'READ';
				if(($permission & self::PERMISSION_UNSET) != 0) $perms[] = 'UNSET';
				if(($permission & self::PERMISSION_WRITE) != 0) $perms[] = 'WRITE';
				
				return $this->_retError(false, $code, sprintf('Permission denied. This action needs %s permission.', join(' and ', $perms)));
			}
			
			return true;
		}
	// }
}

