<?php

/*******************************************************************************

 WINBINDER - A native Windows binding for PHP

 Copyright  2004-2006 Hypervisual - see LICENSE.TXT for details
 Author: Rubem Pechansky (http://www.hypervisual.com/winbinder/contact.php)

 Main inclusion file for WinBinder

*******************************************************************************/

$supported_php_version = preg_match(
    '"^(' . '4\.(3\.(10|11)|4\..)' . '|' . '5\.(0\.[3-5]|[1-3]\..)' . ')$"', PHP_VERSION
);

if (!$supported_php_version)
	die("WinBinder does only support the following PHP versions:\n"
		. "- 4.3.10 up to excluding 5.0.0\n- 5.0.3  up to excluding 6.0.0\n");

if(!extension_loaded('winbinder'))
	if(!dl('php_winbinder.dll'))
		trigger_error("WinBinder extension could not be loaded.\n", E_USER_ERROR);

//------------------------------------------------------------------ END OF FILE



//------------------------------------------------------------------ wb_windows.inc.php
	//-------------------------------------------------------------------- CONSTANTS

	// Windows constants

	define("BM_SETCHECK",			241);
	define("LVM_FIRST",				0x1000);
	define("LVM_DELETEALLITEMS",	(LVM_FIRST+9));
	define("LVM_GETITEMCOUNT",		(LVM_FIRST+4));
	define("LVM_GETITEMSTATE",		(LVM_FIRST+44));
	define("LVM_GETSELECTEDCOUNT",	(LVM_FIRST+50));
	define("LVIS_SELECTED",			2);
	define("TCM_GETCURSEL",			4875);
	define("CB_FINDSTRINGEXACT",	344);
	define("CB_SETCURSEL",			334);
	define("LB_FINDSTRINGEXACT",	418);
	define("LB_SETCURSEL",			390);
	define("TCM_SETCURSEL",			4876);
	define("WM_SETTEXT",			12);

	//------------------------------------------------------------- WINDOW FUNCTIONS

	/*

	Creates a window control, menu, toolbar, status bar or accelerator.

	*/

	function wb_create_control($parent, $class, $caption="", $xpos=0, $ypos=0, $width=0, $height=0, $id=null, $style=0, $lparam=null, $ntab=0)
	{
		switch($class) {

			case Accel:
				return wbtemp_set_accel_table($parent, $caption);

			case ToolBar:
				return wbtemp_create_toolbar($parent, $caption, $width, $height, $lparam);

			case Menu:
				return wbtemp_create_menu($parent, $caption);

			case HyperLink:
				return wbtemp_create_control($parent, $class, $caption, $xpos, $ypos, $width, $height, $id, $style,
				  is_null($lparam) ? NOCOLOR : $lparam, $ntab);

			default:
				return wbtemp_create_control($parent, $class, $caption, $xpos, $ypos, $width, $height, $id, $style, $lparam, $ntab);
		}
	}

	/*

	Sets the value of a control or control item

	*/

	function wb_set_value($ctrl, $value, $item = null)
	{
		if(!$ctrl)
			return null;

		$class = wb_get_class($ctrl);
		switch($class) {

			case ListView:		// Array with items to be checked

				if($value === null)
					break;
				elseif(is_string($value) && strstr($value, ","))
					$values = explode(",", $value);
				elseif(!is_array($value))
					$values = array($value);
				else
					$values = $value;
				foreach($values as $index)
					wbtemp_set_listview_item_checked($ctrl, $index, 1);
				break;

			case TreeView:		// Array with items to be checked

				if($item === null)
					$item = wb_get_selected($ctrl);
				return wbtemp_set_treeview_item_value($ctrl, $item, $value);

			default:

				if($value !== null) {
					return wbtemp_set_value($ctrl, $value, $item);
				}
		}
	}


	/*

	Gets the text from a control, a control item, or a control sub-item.

	*/

	function wb_get_text($ctrl, $item=null, $subitem=null)
	{
		if(!$ctrl)
			return null;

		if(wb_get_class($ctrl) == ListView) {

			if($item !== null) {		// Valid item

				$line = wbtemp_get_listview_text($ctrl, $item);
				if($subitem === null)
					return $line;
				else
					return $line[$subitem];

			} else {					// NULL item

				$sel = wb_get_selected($ctrl);
				if($sel === null) {		// Returns the entire table
					$items = array();
					for($i = 0;;$i++) {
						$item = wbtemp_get_listview_text($ctrl, $i);
						$all = '';
						foreach($item as $col)
							$all .= $col;
						if($all == '')
							break;
						$items[] = $item;
					}
					return $items ? $items : null;
				} else {
					$items = array();
					foreach($sel as $row)
						$items[] = wbtemp_get_listview_text($ctrl, $row);
					return $items ? $items : null;
				}
			}

		} elseif(wb_get_class($ctrl) == TreeView) {

			if($item) {
				return wbtemp_get_treeview_item_text($ctrl, $item);
			} else {
				$sel = wb_get_selected($ctrl);
				if($sel === null)
					return null;
				else {
					return wbtemp_get_text($ctrl);
				}
			}

		} else

			return wbtemp_get_text($ctrl);
	}

	/*

	Sets the text of a control.
	In a ListView, it creates columns: each element of the array text is a column.
	In a tab control, it renames the tabs.
	Sets the text of a control item.

	*/

	function wb_set_text($ctrl, $text, $item=null, $subitem=null)
	{
		if(!$ctrl)
			return null;

		switch(wb_get_class($ctrl)) {

			case ListView:

				if($item !== null) {

					if(!is_array($text) && $subitem !== null) {

						// Set text of a ListView cell according to $item and $subitem

						wbtemp_set_listview_item_text($ctrl, $item, $subitem, $text);

					} else {

						// Set text of several ListView cells, ignoring $subitem

						for($sub = 0; $sub < count($text); $sub++) {
							if($text) {
								if(($text[$sub] !== null)) {
									wbtemp_set_listview_item_text($ctrl, $item, $sub, (string)$text[$sub]);
								}
							} else {
								wbtemp_set_listview_item_text($ctrl, $item, $sub, "");
							}
						}
					}

				} else {

					if(!is_array($text))
						$text = explode(",", $text);

					wb_delete_items($ctrl, null);

					if(!$item) {
						wbtemp_clear_listview_columns($ctrl);

						// Create column headers
						// In the loop below, passing -1 as the last argument of wbtemp_create_listview_column()
						// makes it calculate the column width automatically

						for($i = 0; $i < count($text); $i++) {
							if(is_array($text[$i]))
								wbtemp_create_listview_column($ctrl, $i,
								  (string)$text[$i][0],
								  isset($text[$i][1]) ? (int)$text[$i][1] : -1, -1);
							else
								wbtemp_create_listview_column($ctrl, $i,
								  (string)$text[$i], -1, -1);
						}
					}
				}
				break;

			case ListBox:

				if(!$text) {
					wb_delete_items($ctrl);
				} elseif(is_string($text)) {
					if(strchr($text, "\r") || strchr($text, "\n")) {
						$text = preg_split("/[\r\n,]/", $text);
						wb_delete_items($ctrl);
						foreach($text as $str)
							wbtemp_create_item($ctrl, (string)$str);
					} else {
						$index = wb_send_message($ctrl, LB_FINDSTRINGEXACT, -1, wb_get_address($text));
						wb_send_message($ctrl, LB_SETCURSEL, $index, 0);
					}
				} elseif(is_array($text)) {
					wb_delete_items($ctrl);
					foreach($text as $str)
						wbtemp_create_item($ctrl, (string)$str);
				}
				return;

			case ComboBox:

				if(!$text)
					wb_delete_items($ctrl);
				elseif(is_string($text)) {
					if(strchr($text, "\r") || strchr($text, "\n")) {
						$text = preg_split("/[\r\n,]/", $text);
						wb_delete_items($ctrl);
						foreach($text as $str)
							wbtemp_create_item($ctrl, (string)$str);
					} else {
						$index = wb_send_message($ctrl, CB_FINDSTRINGEXACT, -1, wb_get_address($text));
						wb_send_message($ctrl, CB_SETCURSEL, $index, 0);
						if($index == -1)
							wb_send_message($ctrl, WM_SETTEXT, 0, wb_get_address($text));
					}
				} elseif(is_array($text)) {
					wb_delete_items($ctrl);
					foreach($text as $str)
						wbtemp_create_item($ctrl, (string)$str);
				}
				return;

			case TreeView:

				if($item)
					return wbtemp_set_treeview_item_text($ctrl, $item, $text);
				else
					return wb_create_items($ctrl, $text, true);

			default:
				if(is_array($text))
					return null;
				$text = str_replace("\r", "", (string)$text);
				$text = str_replace("\n", "\r\n", $text);
				return wbtemp_set_text($ctrl, $text, $item);
		}
	}

	/*

	Selects one or more items. Compare with wb_set_value() which checks items instead.

	*/

	function wb_set_selected($ctrl, $selitems, $selected=TRUE)
	{
		switch(wb_get_class($ctrl)) {

			case ComboBox:
				wb_send_message($ctrl, CB_SETCURSEL, (int)$selitems, 0);
				break;

			case ListBox:
				wb_send_message($ctrl, LB_SETCURSEL, (int)$selitems, 0);
				break;

			case ListView:

				if(is_null($selitems)) {
					return wbtemp_select_all_listview_items($ctrl, false);
				} elseif(is_array($selitems)) {
					foreach($selitems as $item)
						wbtemp_select_listview_item($ctrl, $item, $selected);
					return TRUE;
				} else
					return wbtemp_select_listview_item($ctrl, $selitems, $selected);
				break;

			case Menu:
				return wbtemp_set_menu_item_checked($ctrl, $selitems, $selected);

			case TabControl:
				wbtemp_select_tab($ctrl, (int)$selitems);
				break;

			case TreeView:
				wbtemp_set_treeview_item_selected($ctrl, $selitems);
				break;

			default:
				return false;
		}
		return true;
	}

	/*

	Creates one or more items in a control.

	*/

	function wb_create_items($ctrl, $items, $clear=false, $param=null)
	{
		switch(wb_get_class($ctrl)) {

			case ListView:

				if($clear)
					wb_send_message($ctrl, LVM_DELETEALLITEMS, 0, 0);

				$last = -1;

				// For each row

				for($i = 0; $i < count($items); $i++) {
					if(!is_scalar($items[$i]))
						$last = wbtemp_create_listview_item(
							$ctrl, -1, -1, (string)$items[$i][0]);
					else
						$last = wbtemp_create_listview_item(
							$ctrl, -1, -1, (string)$items[$i]);
					wbtemp_set_listview_item_text($ctrl, -1, 0, (string)$items[$i][0]);

					// For each column except the first

					for($sub = 0; $sub < count($items[$i]) - 1; $sub++) {
						if($param) {
							$result = call_user_func($param, 	// Callback function
								$items[$i][$sub + 1],			// Item value
								$i,								// Row
								$sub							// Column
							);
							wbtemp_set_listview_item_text($ctrl, $last, $sub + 1, $result);
						} else
							wbtemp_set_listview_item_text($ctrl, $last, $sub + 1, (string)$items[$i][$sub + 1]);
					}
				}
				return $last;
				break;

			case TreeView:

				if($clear)
					$handle = wb_delete_items($ctrl); // Empty the treeview

				if(!$items)
					break;
				$ret = array();
				for($i = 0; $i < count($items); $i++) {
					$ret[] = wbtemp_create_treeview_item($ctrl,
					  (string)$items[$i][0],	// Name
					  isset($items[$i][1]) ? $items[$i][1] : 0,			// Value
					  isset($items[$i][2]) ? $items[$i][2] : 0,			// Where
					  isset($items[$i][3]) ? $items[$i][3] : -1,			// ImageIndex
					  isset($items[$i][4]) ? $items[$i][4] : -1,			// SelectedImageIndex
					  isset($items[$i][5]) ? $items[$i][5] : 0			// InsertionType
					);
				}
				return (count($ret) > 1 ? $ret : $ret[0]);
				break;

	/*		case ListBox:
			case ComboBox:
				return wb_set_text($ctrl, $items, false);*/

			default:

				if(is_array($items)) {
					foreach($items as $item)
						wbtemp_create_item($ctrl, $item);
					return true;
				} else
					return wbtemp_create_item($ctrl, $items);
				break;
		}
	}

	/*

	Opens the standard Open dialog box.

	*/

	function wb_sys_dlg_open($parent=null, $title=null, $filter=null, $path=null, $filename=null)
	{
		$filter = _make_file_filter($filter ? $filter : $filename);
		return wbtemp_sys_dlg_open($parent, $title, $filter, $path);
	}

	/*

	Opens the standard Save As dialog box.

	*/

	function wb_sys_dlg_save($parent=null, $title=null, $filter=null, $path=null, $filename=null, $defext=null)
	{
		$filter = _make_file_filter($filter ? $filter : $filename);

		return wbtemp_sys_dlg_save($parent, $title, $filter, $path, $filename, $defext);
	}

	//----------------------------------------- AUXILIARY FUNCTIONS FOR INTERNAL USE

	/*

	Creates a file filter for Open/Save dialog boxes based on an array.

	*/

	function _make_file_filter($filter)
	{
		if(!$filter)
			return "All Files (*.*)\0*.*\0\0";

		if(is_array($filter)) {
			$result = "";
			foreach($filter as $line)
				$result .= "$line[0] ($line[1])\0$line[1]\0";
			$result .= "\0";
			return $result;
		} else
			return $filter;
	}
//------------------------------------------------------------------ END OF FILE



//------------------------------------------------------------------ wb_generic.inc.php

	/* Returns an array with all files of subdirectory $path. If $subdirs is TRUE,
	  includes subdirectories recursively. $mask is a PCRE regular expression.
	*/

	function get_folder_files($path, $subdirs=false, $fullname=true, $mask="", $forcelowercase=TRUE)
	{
		// Correct path name, if needed

		$path = str_replace('/', '\\', $path);
		if(substr($path, -1) != '\\')
			$path .= "\\";
		if(!$path || !@is_dir($path))
			return array();

		// Browse the subdiretory list recursively

		$dir = array();
		if($handle = opendir($path)) {
			while(($file = readdir($handle)) !== false) {
				if(!is_dir($path.$file)) {	// No directories / subdirectories
					if($forcelowercase)
						$file = strtolower($file);
					if(!$mask) {
						$dir[] = $fullname ? $path.$file : $file;
					} else if($mask && preg_match($mask, $file)) {
						$dir[] = $fullname ? $path.$file : $file;
					}
				} else if($subdirs && $file[0] != ".") {	// Exclude "." and ".."
					$dir = array_merge($dir, get_folder_files($path.$file, $subdirs, $fullname, $mask));
				}
			}
		}
		closedir($handle);
		return $dir;
	}

	//-------------------------------------------------------------------- INI FILES

	/* Transforms the array $data in a text that can be saved as an INI file.
	  Escapes double-quotes as (\") */

	function generate_ini($data, $comments="")
	{
		if(!is_array($data)) {
			trigger_error(__FUNCTION__ . ": Cannot save INI file.");
			return null;
		}
		$text = $comments;
		foreach($data as $name=>$section) {
			$text .= "\r\n[$name]\r\n";

			foreach($section as $key=>$value) {
				$value = trim($value);
				if((string)((int)$value) == (string)$value)			// Integer: does nothing
					;
				elseif((string)((float)$value) == (string)$value)	// Floating point: does nothing
					;
				elseif($value === "")								// Empty string
					$value = '""';
				elseif(strstr($value, '"'))							// Escape double-quotes
					$value = '"' . str_replace('"', '\"', $value) . '"';
				else
					$value = '"' . $value . '"';

				$text .= "$key = " . $value . "\r\n";
			}
		}
		return $text;
	}

	/*

	Replaces function parse_ini_file() so INI files may be processed more similarly to Windows.
	Replaces escaped double-quotes (\") with double-quotes ("). See manual for details.

	*/

	function parse_ini($initext, $changecase=TRUE, $convertwords=TRUE)
	{
		$ini = preg_split("/\r\n|\n/", $initext);
		$secpattern = "/^\[(.[^\]]*)\]/i";
	//	$entrypattern = "/^([a-z_0-9]*)\s*=\s*\"?([^\"]*)?\"?" . '$' . "/i";
	//	$strpattern = "/^\"?(.[^\"]*)\"?" . '$' . "/i";
		$entrypattern = "/^([a-z_0-9]*)\s*=\s*\"?([^\"]*)?\"?\$/i";
		$strpattern = "/^\"?(.[^\"]*)\"?\$/i";

		$section = array();
		$sec = "";

		// Predefined words

		static $words  = array("yes", "on", "true", "no", "off", "false", "null");
		static $values = array(   1,    1,      1,    0,     0,       0,   null);

		// Lines loop

		for($i = 0; $i < count($ini); $i++) {

			$line = trim($ini[$i]);

			// Replaces escaped double-quotes (\") with special signal /%quote%/

			if(strstr($line, '\"'))
				$line = str_replace('\"', '/%quote%/', $line);

			// Skips blank lines and comments

			if($line == "" || preg_match("/^;/i", $line))
				continue;

			if(preg_match($secpattern, $line, $matches)) {

				// It's a section

				$sec = $matches[1];

				if($changecase)
					$sec = ucfirst(strtolower($sec));

				$section[$sec] = array();

			} elseif(preg_match($entrypattern, $line, $matches)) {

				// It's an entry

				$entry = $matches[1];

				if($changecase)
					$entry = strtolower($entry);

				$value = preg_replace($entrypattern, "\\2", $line);

				// Restores double-quotes (")

				$value = str_replace('/%quote%/', '"', $value);

				// Convert some special words to their respective values

				if($convertwords) {
					$index = array_search(strtolower($value), $words);
					if($index !== false)
						$value = $values[$index];
				}

				$section[$sec][$entry] = $value;

			} else {

				// It's a normal string

				$section[$sec][] = preg_replace($strpattern, "\\1", $line);

			}
		}
		return $section;
	}

//------------------------------------------------------------------ END OF FILE



//------------------------------------------------------------------ wb_resources.inc.php
	//-------------------------------------------------------------------- CONSTANTS

	define("WB_KX_SCREEN", 1.498); // Determined through trial and error
	define("WB_KY_SCREEN", 1.625); // Determined through trial and error

	//------------------------------------------------- WINDOWS CONSTANTS FROM WIN.H

	define("WS_VISIBLE",			0x10000000);
	define("WS_DISABLED",			0x08000000);
	define("WS_GROUP",				0x00020000);
	define("WS_EX_STATICEDGE",		0x00020000);

	// Button styles

	define("BS_PUSHBUTTON",			0x00);
	define("BS_CHECKBOX",			0x02);
	define("BS_AUTOCHECKBOX",		0x03);
	define("BS_RADIOBUTTON",		0x04);
	define("BS_GROUPBOX",			0x07);
	define("BS_AUTORADIOBUTTON",	0x09);
	define("BS_ICON",				0x40);
	define("BS_BITMAP",				0x80);

	// Edit control styles

	define("ES_NUMBER",				0x2000);
	define("ES_PASSWORD",			0x20);
	define("ES_READONLY",			0x0800);
	define("ES_UPPERCASE",			0x08);
	define("ES_LEFT",				0x0);
	define("ES_CENTER",				0x01);
	define("ES_RIGHT",				0x02);
	define("ES_MULTILINE",			0x04);

	// Static styles

	define("SS_LEFT",				0x00);
	define("SS_CENTER",				0x01);
	define("SS_RIGHT",				0x02);
	define("SS_ETCHEDHORZ",			0x10);
	define("SS_ETCHEDVERT",			0x11);
	define("SS_ETCHEDFRAME",		0x12);
	define("SS_ICON",				0x03);
	define("SS_BITMAP",				0x0E);
	define("SS_LEFTNOWORDWRAP",		0x0C);
	define("SS_WORDELLIPSIS",		0xC000);

	// Other styles

	define("CBS_SORT",				0x100);
	define("CBS_DROPDOWNLIST",		3);

	define("LBS_SORT",				2);
	define("LVS_NOSORTHEADER",		0x00008000);
	define("LVS_GRIDLINES",			0x00800000);	// Actually WS_BORDER
	define("LVS_CHECKBOXES",		0x00000800);	// Actually LVS_ALIGNLEFT
	define("LVS_SINGLESEL",			0x00000004);
	define("TBS_AUTOTICKS",			1);

	//-------------------------------------------------------------------- FUNCTIONS

	/*

	  Returns the WinBinder code that results from the resource text $rc, usually
	   read from a RC (Windows resource) file.

	TODO: Extend support to several RC generators (currently supports WinAsm Studio only)
	NOTE: Caption is not used, it's taken from the resource instead. The parameter is kept
	      here just to be compatible with wb_create_window()
	*/

	function parse_rc($rc, $winvar='$mainwin', $parent=null, $type="AppWindow", $caption=null,
	  $x=WBC_CENTER, $y=WBC_CENTER, $width=WBC_CENTER, $height=WBC_CENTER, $style=0, $lparam=0,
	  $respath=PATH_RES)
	{
		global $_winclass, $_usergeom, $path_res;

		// Read file

		$_usergeom = array($x, $y, $width, $height);
		$path_res = $respath;

		// Remove comments and useless spaces

		$rc = preg_replace("/^\s*;.*$/m", "", $rc);
		$rc = preg_replace("/^\s*(.*)$/m", "\\1", $rc);

		// Maintain #defines and discard the rest (fixed to work with newer versions of PHP -- thanks Hans)

	//	$def = preg_replace("/(?!^\s*#define)(.*)$/m", "\\2", $rc);
		$def = preg_replace('/^((?!#define).)*$/m', "\\2", $rc);

		// Remove blank lines

		$def = preg_replace("/\n+/m", "\n", $def);

		// Change string C #defines to PHP format

		$def = preg_replace("/#define\s+(\w+)\s+\"(.*)\"/", "if(!defined(\"\\1\")) define(\"\\1\", \"\\2\");", $def);

		// Change character C #defines to PHP format

		$def = preg_replace("/#define\s+(\w+)\s+'(.+)'/", "if(!defined(\"\\1\")) define(\"\\1\", \"\\2\");", $def);

		// Change numeric C #defines to PHP format

		$def = preg_replace("/#define\s+(\w+)\s+(\S+)/", "if(!defined(\"\\1\")) define(\"\\1\", \\2);", $def);
		$def = "// Control identifiers\n\n" . preg_replace("/(\r\n|\r|\n)+/sm", "\n", $def);

		// Return to original string and eliminates the #defines

		$rc = preg_replace("/^\s*#define(.*)$/m", "", $rc);

		// Create the window

		$_winclass = $type;

		$tok = "\s*((?:[\"'][\S \t]*[\"'])|(?:[\S^,'\"]+))\s*";	// Normal or quoted token
		$rc = "// Create window\n\n" . preg_replace_callback("/^$tok\s+DIALOGEX$tok,$tok,$tok,$tok\s+CAPTION$tok\s+FONT$tok,$tok\s+STYLE$tok\s+EXSTYLE$tok/m", "_scale_dialog", $rc);

		// Create the controls

		$rc = preg_replace_callback("/^\s*CONTROL\s+$tok,$tok,$tok,$tok,$tok,$tok,$tok,$tok,$tok/m", "_scale_controls", $rc);

		// Create BEGIN / END comments

		$rc = preg_replace("/^\s*BEGIN/m", "\n// Insert controls\n", $rc);
		$rc = preg_replace("/^\s*END/m", "\n// End controls", $rc);

		// Replace variable names

		$rc = str_replace("%WINVAR%", $winvar, $rc);
		$rc = str_replace("%PARENT%", $parent? $parent : "NULL", $rc);
		$rc = str_replace("%STYLE%",  $style, $rc);
		$rc = str_replace("%LPARAM%", $lparam, $rc);

		return "$def\n$rc";
	}

	//----------------------------------------------------------- INTERNAL FUNCTIONS

	function _scale_dialog($c)
	{
		global $_winclass, $_usergeom, $_tabN;

		if($_winclass == "TabControl") {

			$_tabN++;
			$code = "wbtemp_create_item(%PARENT%, ". $c[6] . ");\n";

		} else {

			$_addx = 8;					//width + 2xborder
			$_addy = 4 + 42 + 17 + 4;	//border + caption + border

			switch(is_string($_winclass) ? strtolower($_winclass) : $_winclass) {

				case "appwindow":
					$_winclass = AppWindow;
					$_addx = 8;					//width + 2xborder
					$_addy = 3 + 18 + 22 + 18 + 3;	//border + caption + menu + statusbar + border
					break;
				case "resizablewindow":
					$_winclass = ResizableWindow;
					$_addx = 8;					//width + 2xborder
					$_addy = 4 + 42 + 17 + 4;	//border + caption + menu + statusbar + border
					break;
				case "modaldialog":
					$_winclass = ModalDialog;
					$_addx = 8;					//width + 2xborder
					$_addy = 4 + 42 + 17 + 4;	//border + caption + border
					break;
				case "modelessdialog":
					$_winclass = ModelessDialog;
					break;
				case "tooldialog":
					$_winclass = ToolDialog;
					break;
			}

			if(!(($_usergeom[0] == WBC_CENTER && $_usergeom[1] == WBC_CENTER &&
			  $_usergeom[2] == WBC_CENTER && $_usergeom[3] == WBC_CENTER))) {

				$code = "%WINVAR% = wb_create_window(" .
				"%PARENT%, " .								// parent
				"$_winclass, " .							// class
				$c[6] . ", " .								// caption
				$_usergeom[0] . ", " .						// left
				$_usergeom[1] . ", " .						// top
				$_usergeom[2] . ", " .						// width
				$_usergeom[3] . ", " .						// height
				"%STYLE%, " . 								// style
				"%LPARAM%);\n";								// lparam

			} else {

				if(is_array($_usergeom)) {
					if(count($_usergeom) == 2) {		// Width, height only
						$_usergeom[2] = $_usergeom[0];
						$_usergeom[3] = $_usergeom[1];
						$_usergeom[0] = WBC_CENTER;
						$_usergeom[1] = WBC_CENTER;
					}
				} elseif(is_null($_usergeom)) {
					$_usergeom[0] = WBC_DEFAULTPOS;
					$_usergeom[1] = WBC_DEFAULTPOS;
					$_usergeom[2] = WBC_DEFAULTPOS;
					$_usergeom[3] = WBC_DEFAULTPOS;
				}

				$code = "%WINVAR% = wb_create_window(" .
				"%PARENT%, " .								// parent
				"$_winclass, " .							// class
				$c[6] . ", " .								// caption
				"WBC_CENTER, " .							// left
				"WBC_CENTER, " .							// top
	//			(int)($c[4] * WB_KX_SCREEN + $_addx) . ", " .
	//			(int)($c[5] * WB_KY_SCREEN + $_addy) . ", " .
				(int)($c[4] * WB_KX_SCREEN) . ", " .
				(int)($c[5] * WB_KY_SCREEN) . ", " .
				"%STYLE%, " .								// style
				"%LPARAM%);\n";								// lparam
			}

			$_tabN = 0;

		}
		return $code;
	}

	function _scale_controls($c)
	{
		global $_tabN, $path_res;

		$winclass = $c[3];
		$winstyle = hexdec($c[4]);
		$winexstyle = hexdec($c[9]);

		if(_bit_test($winstyle, WS_VISIBLE))
			$style = "WBC_VISIBLE";
		else
			$style = "WBC_INVISIBLE";

		if(_bit_test($winstyle, WS_DISABLED))
			$style .= " | WBC_DISABLED";
		else
			$style .= " | WBC_ENABLED";

		if(_bit_test($winexstyle, WS_EX_STATICEDGE))
			$style .= " | WBC_BORDER";

		// Set attributes according to control class

		switch(strtolower($winclass)) {

			case '"button"':

				switch($winstyle & 0x0F) {
					case BS_AUTORADIOBUTTON:
					case BS_RADIOBUTTON:
						$class = "RadioButton";
						if(_bit_test($winstyle, WS_GROUP))
							$style .= " | WBC_GROUP";
						break;
					case BS_AUTOCHECKBOX:
					case BS_CHECKBOX:
						$class = "CheckBox";
						break;
					case BS_GROUPBOX:
						$class = "Frame";
						break;
					case BS_PUSHBUTTON:
					default:
						$class = "PushButton";
						break;
				}
				break;

			case '"static"':

				switch($winstyle & 0x1F) {
					case SS_ICON:
					case SS_BITMAP:
						$style .= " | WBC_IMAGE | WBC_CENTER";
						$class = "Frame";
						break;
					case SS_ETCHEDHORZ:
					case SS_ETCHEDVERT:
					case SS_ETCHEDFRAME:
						$class = "Frame";
						break;
					case SS_CENTER:
						if(_bit_test($winstyle, SS_WORDELLIPSIS))
							$style .= " | WBC_ELLIPSIS";
						$style .= " | WBC_CENTER";
						$class = "Label";
						break;
					case SS_RIGHT:
						if(_bit_test($winstyle, SS_WORDELLIPSIS))
							$style .= " | WBC_ELLIPSIS";
						$style .= " | WBC_RIGHT";
						$class = "Label";
						break;
					case SS_LEFT:
					default:
						if(!_bit_test($winstyle, SS_LEFTNOWORDWRAP))
							$style .= " | WBC_MULTILINE";
						if(_bit_test($winstyle, SS_WORDELLIPSIS))
							$style .= " | WBC_ELLIPSIS";
						$class = "Label";
						break;
				}
				break;

			case '"edit"':
				$class = "EditBox";
				if(_bit_test($winstyle, ES_MULTILINE)) {
					$style .= " | WBC_MULTILINE";
				} else {
					switch($winstyle & 0x03) {
						case ES_CENTER:
							$style .= " | WBC_CENTER";
							break;
						case ES_RIGHT:
							$style .= " | WBC_RIGHT";
							break;
						case ES_LEFT:
						default:
							break;
					}
				}
				if(_bit_test($winstyle, ES_READONLY))
					$style .= " | WBC_READONLY";
				if(_bit_test($winstyle, ES_PASSWORD))
					$style .= " | WBC_MASKED";
				if(_bit_test($winstyle, ES_NUMBER))
					$style .= " | WBC_NUMBER";
				break;

			case '"richedit20a"':
				if(_bit_test($winstyle, ES_READONLY))
					$style .= " | WBC_READONLY";
				$class = "RTFEditBox";
				switch($winstyle & 0x03) {
					case ES_CENTER:
						$style .= " | WBC_CENTER";
						break;
					case ES_RIGHT:
						$style .= " | WBC_RIGHT";
						break;
					case ES_LEFT:
					default:
						break;
				}
				break;

			case '"combobox"':
				$class = "ComboBox";
				if(_bit_test($winstyle, CBS_SORT))
					$style .= " | WBC_SORT";
				if(_bit_test($winstyle, CBS_DROPDOWNLIST))
					$style .= " | WBC_READONLY";
				break;

			case '"listbox"':
				$class = "ListBox";
				if(_bit_test($winstyle, LBS_SORT))
					$style .= " | WBC_SORT";
				break;

			case '"scrollbar"':
				$class = "ScrollBar";
				break;

			case '"syslistview32"':
				$class = "ListView";
				if(!_bit_test($winstyle, LVS_NOSORTHEADER))
					$style .= " | WBC_SORT";
				if(_bit_test($winstyle, LVS_GRIDLINES))
					$style .= " | WBC_LINES";
				if(_bit_test($winstyle, LVS_CHECKBOXES))
					$style .= " | WBC_CHECKBOXES";
				if(!_bit_test($winstyle, LVS_SINGLESEL))
					$style .= " | WBC_SINGLE";
				break;

			case '"systabcontrol32"':
				$class = "TabControl";
				break;

			case '"systreeview32"':
				$class = "TreeView";
				break;

			case '"toolbarwindow32"':
				$class = "ToolBar";
				break;

			case '"msctls_progress32"':
				$class = "Gauge";
				break;

			case '"msctls_statusbar32"':
				$class = "StatusBar";
				break;

			case '"sysmonthcal32"':
				$class = "Calendar";
				break;

			case '"msctls_trackbar32"':
				$class = "Slider";
				if(_bit_test($winstyle, TBS_AUTOTICKS))
					$style .= " | WBC_LINES";
				break;

			case '"msctls_updown32"':
				$class = "Spinner";
				if(_bit_test($winstyle, WS_GROUP))
					$style .= " | WBC_GROUP";
				break;
		}

		// Convert Windows style to WinBinder style

		$str = "wb_create_control(" .
			"%WINVAR%, " .								// Parent
			$class . ", " . 							// Class
			$c[1] . ", " .								// Caption
			(int)($c[5] * WB_KX_SCREEN) . ", " .		// Left
			(int)($c[6] * WB_KY_SCREEN) . ", " .		// Top
			(int)($c[7] * WB_KX_SCREEN) . ", " .		// Width
			(int)($c[8] * WB_KY_SCREEN) . ", " .		// Height
			$c[2] . ", " . 								// ID
			$style . ", " .			 					// Style
			"0" .										// Param
			($_tabN ? ", " . ($_tabN - 1) . ");\n" : ");\n");	// Tab #

		// Add some attributes to controls where needed

		switch($class) {

			case "Frame":

				if(strstr($style, "WBC_IMAGE")) {
					if(($winstyle & (SS_BITMAP | SS_ICON)) && ($c[1] !== '""')) {
						$image = $path_res . _trim_quotes($c[1]);
						if(preg_match("/\.(bmp|ico)$/", $image))
							$str = "\$_tmp_ctrl_ = " . $str . "wb_set_image(\$_tmp_ctrl_, '$image', GREEN);" . " unset(\$_tmp_ctrl_);\n";
					}
				}
				break;

			case "PushButton":

				if(($winstyle & (BS_BITMAP | BS_ICON)) && ($c[1] !== '""')) {
					$image = $path_res . _trim_quotes($c[1]);
					if($image)
						if(preg_match("/\.(bmp|ico)$/", $image))
							$str = "\$_tmp_ctrl_ = " . $str . "wb_set_image(\$_tmp_ctrl_, '$image', GREEN);" . " unset(\$_tmp_ctrl_);\n";
				}
				break;
		}

		return $str;
	}

	function _trim_quotes($str)
	{
		return str_replace('"', '', $str);
	}

	function _bit_test($v, $t)
	{
		return (($v & $t) == $t);
	}
//------------------------------------------------------------------ END OF FILE


?>
